package yrpg.application.crmvisit.plugin.form;

import kd.bos.base.BaseShowParameter;
import kd.bos.base.MobileBaseShowParameter;
import kd.bos.bill.AbstractMobBillPlugIn;
import kd.bos.bill.MobileBillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.ext.form.control.CustomControl;
import kd.bos.form.CloseCallBack;
import kd.bos.form.ShowType;
import kd.bos.form.control.Button;
import kd.bos.form.control.Control;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.events.CustomEventArgs;
import kd.bos.form.events.HyperLinkClickEvent;
import kd.bos.form.events.HyperLinkClickListener;
import kd.bos.form.events.SetFilterEvent;
import kd.bos.form.events.SetFilterListener;
import kd.bos.list.BillList;
import kd.bos.list.MobileListShowParameter;
import kd.bos.list.events.ListHyperLinkClickEvent;
import kd.bos.list.events.ListHyperLinkClickListener;
import kd.bos.list.events.ListRowClickEvent;
import kd.bos.list.events.ListRowClickListener;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;
import kd.sdk.plugin.Plugin;
import kd.bos.form.control.Label;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;

import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 单据界面插件(移动端)
 */
public class VisitEditMobBillPlugin extends AbstractMobBillPlugIn implements Plugin, SetFilterListener,HyperLinkClickListener{

    //表单标识
    private static String VISIT = "yrpg_crm_visit";

    @Override
    public void initialize() {
        super.initialize();
        BillList billList = this.getView().getControl("yrpg_billlistap");
        if (billList != null) {
            billList.addSetFilterListener(this);//添加filter监听
        }
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        //新建拜访，专家维护
        this.addClickListeners("yrpg_add_visit","yrpg_expert_apply","yrpg_expertdetails");
        BillList list = getControl("yrpg_billlistap");
        list.addListRowClickListener(new ListRowClickListener() {
            @Override
            public void listRowClick(ListRowClickEvent evt) {
                ListRowClickListener.super.listRowClick(evt);
                Object pk = evt.getCurrentListSelectedRow().getPrimaryKeyValue();
                MobileBillShowParameter showParameter = new MobileBillShowParameter();
                showParameter.setPkId(pk);
                showParameter.setFormId("yrpg_crm_visit_copy_mob");
                showParameter.getOpenStyle().setShowType(ShowType.Floating);
                showParameter.setStatus(OperationStatus.EDIT);
                showParameter.setCloseCallBack(new CloseCallBack(getPluginName(),"yrpg_crm_visit_copy_mob"));
                getView().showForm(showParameter);
            }
        });
    }

    /**
     * 添加过滤条件
     */
    @Override
    public void setFilter(SetFilterEvent paramSetFilterEvent) {
        switch (((BillList) paramSetFilterEvent.getSource()).getKey()) {
            case "yrpg_billlistap"://查询出来的数据库字段
                //当前的拜访
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.HOUR_OF_DAY, 0);
                calendar.set(Calendar.MINUTE, 0);
                calendar.set(Calendar.SECOND, 0);
                calendar.set(Calendar.MILLISECOND, 0);
                Timestamp timestamp = new Timestamp(calendar.getTime().getTime());
                QFilter nowStartDateFilter = new QFilter("yrpg_appointment_time", QCP.equals, timestamp);
                long userId = UserServiceHelper.getCurrentUserId();
                QFilter userFilter = new QFilter("creator", QCP.equals, userId);
//                QFilter qFilter = new QFilter("creator", QCP.equals, UserServiceHelper.getCurrentUserId());
                paramSetFilterEvent.addCustomQFilter(nowStartDateFilter.and(userFilter));
                break;
            default:
        }
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
//        bindTodayVisitList();
        //日历
        long userId = UserServiceHelper.getCurrentUserId();
        Date now = new Date();
        QFilter userFilter = new QFilter("creator", QCP.equals, userId);
        QFilter startDateFilter = new QFilter("yrpg_appointment_time", QCP.large_equals, DateUtils.addDays(now, -30));
        QFilter endDateFilter = new QFilter("yrpg_appointment_time", QCP.less_equals, DateUtils.addDays(now, 30));
        DynamicObjectCollection systemCustomList = QueryServiceHelper.query(VISIT,
                "yrpg_appointment_time", new QFilter[]{userFilter, startDateFilter, endDateFilter});
        Map<String, Object> paramsMap = new HashMap<String, Object>();
        List<String> dateList = new ArrayList<>();
        for (DynamicObject dynamicObject : systemCustomList) {
            dateList.add(dynamicObject.get("yrpg_appointment_time").toString().substring(0, 10));
        }
        paramsMap.put("dateList", dateList);

        CustomControl customcontrol = this.getView().getControl("yrpg_customcontrolap");
        customcontrol.setData(paramsMap);
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
//        bindTodayVisitList();
    }

    @Override
    public void beforeBindData(EventObject e) {
        super.beforeBindData(e);
//        long userId = UserServiceHelper.getCurrentUserId();
//        Date now = new Date();
//        QFilter userFilter = new QFilter("creator", QCP.equals, userId);
//        QFilter startDateFilter = new QFilter("yrpg_appointment_time", QCP.large_equals, DateUtils.addDays(now, -30));
//        QFilter endDateFilter = new QFilter("yrpg_appointment_time", QCP.less_equals, DateUtils.addDays(now, 30));
//        DynamicObjectCollection systemCustomList = QueryServiceHelper.query(VISIT,
//                "yrpg_appointment_time", new QFilter[]{userFilter, startDateFilter, endDateFilter});
//        Map<String, Object> paramsMap = new HashMap<String, Object>();
//        List<String> dateList = new ArrayList<>();
//        for (DynamicObject dynamicObject : systemCustomList) {
//            dateList.add(dynamicObject.get("yrpg_appointment_time").toString().substring(0, 10));
//        }
//        paramsMap.put("dateList", dateList);
//
//        CustomControl customcontrol = this.getView().getControl("yrpg_customcontrolap");
//        customcontrol.setData(paramsMap);
    }

    @Override
    public void customEvent(CustomEventArgs e) {
        String status = this.getView().getFormShowParameter().getStatus().toString();
        if (status.equals("ADDNEW")){
            return;
        }

        // 设计器上自定义控件的标识
        String key = e.getKey();
        // 前端通过model.invoke传给后端的数据
        String args = e.getEventArgs();
        // 前端通过model.invoke定义的事件名
        String eventName = e.getEventName();
        System.out.println("自定义控件标识: " + key + ";事件名: " + eventName + ";自定义控件回传参数: " + args);
        //this.getView().showMessage("自定义控件标识: " + key + ";事件名: " + eventName + ";自定义控件回传参数: " + args);
        switch (eventName) {
            case "selectData":
                String clickDate = args.replaceAll("/", "-") + " 00-00-00";
                SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss");
                Timestamp timestamp = null;
                try {
                    timestamp = new Timestamp(dateFormat.parse(clickDate).getTime());
                } catch (ParseException ex) {
                    throw new RuntimeException(ex);
                }
                bindVisitList(timestamp);
                break;
            case "expertInfo":
                BaseShowParameter baseShowParameter = new BaseShowParameter();
                baseShowParameter.getOpenStyle().setShowType(ShowType.Modal);
                baseShowParameter.setFormId("yrpg_expert_detail");
                baseShowParameter.getOpenStyle().setTargetKey("yrpg_expert_info");
                DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle("\n" +
                        "yrpg_crm_expert", "id,yrpg_expert_name,yrpg_expert_phone", new QFilter[]{});
                baseShowParameter.setPkId(dynamicObject.get("id"));
                getView().showForm(baseShowParameter);
                break;
        }
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        viewExpertApply(evt);
        viewAddVisit(evt);
        viewExpertDetails(evt);
        viewBillList(evt);
    }

    private void viewBillList(EventObject evt) {
        if("yrpg_expertdetails".equals(((Button)evt.getSource()).getKey())==false)
            return;
        BillList billList = getControl("yrpg_billlistap");
        ListSelectedRow listSelectedRow = billList.getCurrentSelectedRowInfo();
        Long pk = (Long) listSelectedRow.getPrimaryKeyValue();
        QFilter filter = new QFilter("id", QCP.equals, pk);
        DynamicObject crmVisit = QueryServiceHelper.queryOne("yrpg_crm_visit_copy","id,yrpg_expert.id",new QFilter[]{filter});
        if (crmVisit!=null){
            MobileBaseShowParameter showParameter = new MobileBaseShowParameter();
            showParameter.setFormId("yrpg_crm_expert_mob");
            showParameter.getOpenStyle().setShowType(ShowType.Modal);
            showParameter.getOpenStyle().setTargetKey("yrpg_expert_info");
            showParameter.setPkId(crmVisit.getLong("yrpg_expert.id"));
            showParameter.setStatus(OperationStatus.VIEW);
            this.getView().showForm(showParameter);
        }
    }

    /**
     * 点击专家详情
     * @param evt
     */
    private void viewExpertDetails(EventObject evt) {
        if("yrpg_billlistap".equals(((Button)evt.getSource()).getKey())==false)
            return;
        BillList billList = getControl("yrpg_billlistap");
        ListSelectedRow listSelectedRow = billList.getCurrentSelectedRowInfo();
        Long pk = (Long) listSelectedRow.getPrimaryKeyValue();
        QFilter filter = new QFilter("id", QCP.equals, pk);
        DynamicObject visitApply = QueryServiceHelper.queryOne("yrpg_crm_visit_copy_mob","id",new QFilter[]{filter});
        if (visitApply!=null){
            MobileBillShowParameter showParameter = new MobileBillShowParameter();
            showParameter.setFormId("yrpg_crm_visit_copy_mob");
            showParameter.getOpenStyle().setShowType(ShowType.Floating);
//            showParameter.getOpenStyle().setTargetKey("yrpg_expert_info");
            showParameter.setPkId(pk);
            this.getView().showForm(showParameter);
        }
    }

    /**
     * 弹专家维护
     * @param evt
     */
    private void viewExpertApply(EventObject evt) {
        if("yrpg_expert_apply".equals(((Button)evt.getSource()).getKey())==false)
            return;
        MobileListShowParameter showParameter = new MobileListShowParameter();
        showParameter.setFormId("bos_moblist");
        showParameter.setBillFormId("yrpg_relationship_bind");
        showParameter.getOpenStyle().setShowType(ShowType.Floating);
        this.getView().showForm(showParameter);

    }

    /**
     * 弹新建拜访
     * @param evt
     */
    private void viewAddVisit(EventObject evt) {
        if("yrpg_add_visit".equals(((Button)evt.getSource()).getKey())==false)
            return;
        MobileBillShowParameter showParameter = new MobileBillShowParameter();
        showParameter.setFormId("yrpg_crm_visit_copy_mob");//布局标识
        showParameter.setCloseCallBack(new CloseCallBack(this.getPluginName(), "yrpg_crm_visit_copy_mob"));
        showParameter.getOpenStyle().setShowType(ShowType.Floating);
        this.getView().showForm(showParameter);
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
        if (StringUtils.equals("yrpg_crm_visit_copy_mob", closedCallBackEvent.getActionId())) {
            this.getView().invokeOperation("refresh");
        }

    }

    private void setListNum(Timestamp timestamp) {
        long userId = UserServiceHelper.getCurrentUserId();
        QFilter userFilter = new QFilter("creator", QCP.equals, userId);
        QFilter dateFilter = new QFilter("yrpg_appointment_time", QCP.equals, timestamp);
        DynamicObjectCollection todayList = QueryServiceHelper.query(VISIT,
                "count(*)", new QFilter[]{userFilter, dateFilter});
        Label label = this.getView().getControl("yrpg_labelap");
        label.setText("(" + todayList.size() + ")");

    }

    private void bindVisitList(Timestamp timestamp){
        MobileListShowParameter listShowParameter = new MobileListShowParameter();
        listShowParameter.getOpenStyle().setShowType(ShowType.InContainer);
        listShowParameter.setFormId("bos_moblist");
        listShowParameter.getOpenStyle().setTargetKey("yrpg_flexpanelap3");
        listShowParameter.setBillFormId("yrpg_crm_visit_copy");
        QFilter nowStartDateFilter = new QFilter("yrpg_appointment_time", QCP.equals, timestamp);
        listShowParameter.getListFilterParameter().getQFilters().add(nowStartDateFilter);
        listShowParameter.setShowFilter(false);
        getView().showForm(listShowParameter);
        setListNum(timestamp);
    }

    private void bindTodayVisitList(){
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Timestamp timestamp = new Timestamp(calendar.getTime().getTime());
        bindVisitList(timestamp);
    }

    @Override
    public void hyperLinkClick(HyperLinkClickEvent e) {
        Control source = (Control)e.getSource();
    }
}